#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <GL/glut.h>
#include <math.h>
#include "xgetopt.h"
#include "obj.h"
#include "version.h"

struct View3D       view3d ;

int     startx ;
int     starty ;
int     keystat ;
int     mouse_Button ;
int     alt_stat ;
int     shift_stat ;
int     cntl_stat ;

char   *inputpose  ;
char   *outputpose ;

char   default_inputpose[] = "input.pose"  ;
char   default_outputpose[]= "output.pose" ;

GLfloat light0pos[] = {   0, 0,   1.0, 0 };


GLfloat s_ctlpoints[48];
GLfloat s_knots[8] = {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};

GLUnurbsObj *theNurb;

extern FILE *yyin ;
extern int   yyerrors ;
int yyparse ();

struct Node node ;

int draw_obj( struct MotionTree *tree, int level ) ;




void idle(void)
{
  glutPostRedisplay();
}

void resize(int w, int h)
{
  glViewport(0, 0, w, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(view3d.fov, (double)view3d.win_w / (double)view3d.win_h, 0.3, 200.0);
  gluLookAt(view3d.v.x, view3d.v.y, view3d.v.z,
	    view3d.c.x, view3d.c.y, view3d.c.z,
	    view3d.u.x, view3d.u.y, view3d.u.z) ;
  glMatrixMode(GL_MODELVIEW);
  view3d.win_w=w ;
  view3d.win_h=h ;
}

void display(void)
{
  
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glLoadIdentity();
  
  glViewport(0, 0, view3d.win_w, view3d.win_h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(view3d.fov, (double)view3d.win_w / (double)view3d.win_h, 0.3, 200.0);
  gluLookAt(view3d.v.x, view3d.v.y, view3d.v.z,
	    view3d.c.x, view3d.c.y, view3d.c.z,
	    view3d.u.x, view3d.u.y, view3d.u.z) ;
  glMatrixMode(GL_MODELVIEW);
    
  glLoadIdentity();
  glLightfv(GL_LIGHT0, GL_POSITION, light0pos);

  { /*** Draw cursor ***/
    glDisable(GL_LIGHTING);
    glPointSize(8.0);
    glColor3f(1.0, 0.0, 0.0);
    glBegin(GL_POINTS);
    glVertex3f(view3d.cur.x, view3d.cur.y, view3d.cur.z) ;
    glEnd();
  }

  glTranslated( view3d.p.x , view3d.p.y , view3d.p.z);
  glRotated( view3d.r.x , 1.0, 0.0, 0.0 );
  glRotated( view3d.r.y , 0.0, 1.0, 0.0 );
  glRotated( view3d.r.z , 0.0, 0.0, 1.0 );

  glBegin( GL_LINES  ) ;
  glColor3f(1.0, 0.0, 0.0);
  glVertex3d( 0.0, 0.0, 0.0 ) ;
  glVertex3d( 2.0, 0.0, 0.0 ) ;
  glColor3f(0.0, 1.0, 0.0);
  glVertex3d( 0.0, 0.0, 0.0 ) ;
  glVertex3d( 0.0, 2.0, 0.0 ) ;
  glColor3f(0.0, 0.0, 1.0);
  glVertex3d( 0.0, 0.0, 0.0 ) ;
  glVertex3d( 0.0, 0.0, 2.0 ) ;
  glEnd();
  
  /*** Draw object ***/
  draw_obj( &M_tree, 0 ) ;
	   
  //glFlush();
  glutSwapBuffers(); /** for DoubleBuffer **/

}  

void key(unsigned char key , int x , int y)
{
  switch( key ){
  case 'r' :
    view3d.v.x=0.0   ;  view3d.v.y=0.0   ;  view3d.v.z=5.0   ;
    view3d.c.x=0.0   ;  view3d.c.y=0.0   ;  view3d.c.z=0.0   ;
    view3d.u.x=0.0   ;  view3d.u.y=1.0   ;  view3d.u.z=0.0   ;
    view3d.r.x=0.0   ;  view3d.r.y=0.0   ;  view3d.r.z=0.0   ;
    view3d.p.x=0.0   ;  view3d.p.y=0.0   ;  view3d.p.z=0.0   ;
    view3d.scale=1.0 ;
    view3d.cur.x=0.0 ;  view3d.cur.y=0.0 ;  view3d.cur.z=0.0 ;
    view3d.sta.x=0.0 ;  view3d.sta.y=0.0 ;  view3d.sta.z=0.0 ;
    glutPostRedisplay();
    break ;
  case 'q' :
    exit(0) ;
    break ;
  case 'o' :
    {
      FILE *ofp ;
      int i ;

      ofp=fopen(outputpose,"w") ;
      for( i=0 ; i<M_tree.num ; i++ ){
	DumpMtNode( ofp, M_tree.node[i], 0 ) ;
      }
      fclose(ofp) ;
    }
    break ;
  default :
    break ;
  }
  //printf("Key = %d\n" , key);
}

void skey(int key , int x , int y)
{

  switch( key ){
  case GLUT_KEY_PAGE_UP :
    break ;
  case GLUT_KEY_PAGE_DOWN :
    break ;
  case 100 : //GULT_KEY_LEFT :
    break ;
  case 102 : //GULT_KEY_RIGHT :
    break ;
  case 101 : //GULT_KEY_UP :
    break ;
  case 103 : //GULT_KEY_DOWN :
    break ;
  default :
    break ;
  }
  //printf("Key = %d\n" , key);
}

void mousebutton(int button, int state, int x, int y)
{
  keystat=glutGetModifiers() ;

  alt_stat  =keystat & GLUT_ACTIVE_ALT ;
  shift_stat=keystat & GLUT_ACTIVE_SHIFT ;
  cntl_stat =keystat & GLUT_ACTIVE_CTRL ;

  if( state==GLUT_DOWN ){
    if( button==GLUT_LEFT_BUTTON ){
      if( (mouse_Button & 1)==0 ){
	struct View3D view_stack=view3d ;
	
    	node.node= NULL ;
	node.d   = -1.0 ;
	view_set( &view_stack ) ;
	search_node( view_stack, &M_tree, &node ) ;
      }
      mouse_Button |= 1 ;
    }
    if( button==GLUT_RIGHT_BUTTON ){
      mouse_Button |= 2 ;
    }
  }
  if( state==GLUT_UP ){
    view3d.cur.x =  view3d.v.z*tan(dec2rad(view3d.fov/2.0)) * (startx     -(view3d.win_w/2))/(view3d.win_h/2) ;
    view3d.cur.y = -view3d.v.z*tan(dec2rad(view3d.fov/2.0)) * (starty     -(view3d.win_h/2))/(view3d.win_h/2) ;
    if( button==GLUT_LEFT_BUTTON ){
      mouse_Button &= ~1 ;
    }
    if( button==GLUT_RIGHT_BUTTON ){
      mouse_Button &= ~2 ;
    }
  }
}

void mousemotion(int x, int y)
{

  alt_stat  =keystat & GLUT_ACTIVE_ALT ;
  shift_stat=keystat & GLUT_ACTIVE_SHIFT ;
  cntl_stat =keystat & GLUT_ACTIVE_CTRL ;

  if( alt_stat!=0 ){
    if(  shift_stat==0 && cntl_stat==0 ){
      view3d.r.y+= startx-x ;
      if( view3d.r.y<0.0   ) view3d.r.y+=360.0 ;
      if( view3d.r.y>360.0 ) view3d.r.y-=360.0 ;
      view3d.r.x-= starty-y ;
      if( view3d.r.x<0.0   ) view3d.r.x+=360.0 ;
      if( view3d.r.x>360.0 ) view3d.r.x-=360.0 ;
      
      startx=x ;
      starty=y ;

      glutPostRedisplay();
    }
    
    if( shift_stat==0 && cntl_stat!=0 ){
      view3d.v.z += (starty-y)/10.0 ;
      if( view3d.v.z> 100 ) view3d.v.z= 100 ;
      if( view3d.v.z< 0.0 ) view3d.v.z= 0.0 ;
      startx=x ;
      starty=y ;
      glutPostRedisplay();
    }

    if( shift_stat!=0 && cntl_stat==0 ){
      view3d.p.x -= (startx-x)/20.0 ;
      view3d.p.y += (starty-y)/20.0 ;
      startx=x ;
      starty=y ;
      glutPostRedisplay();
    }
  }
  
  {
    if( node.node!=NULL ){
      if( cntl_stat==0 ){
	node.node->Mt.r.x+=starty-y ;
	if( node.node->Mt.r.x<0.0   ) node.node->Mt.r.x+=360.0 ;
	if( node.node->Mt.r.x>360.0 ) node.node->Mt.r.x-=360.0 ;
	node.node->Mt.r.y+=startx-x ;
	if( node.node->Mt.r.y<0.0   ) node.node->Mt.r.y+=360.0 ;
	if( node.node->Mt.r.y>360.0 ) node.node->Mt.r.y-=360.0 ;
      }else{
	node.node->Mt.r.z+=startx-x ;
	if( node.node->Mt.r.z<0.0   ) node.node->Mt.r.z+=360.0 ;
	if( node.node->Mt.r.z>360.0 ) node.node->Mt.r.z-=360.0 ;
      }
	
    }
    startx=x ;
    starty=y ;
  }
  
  glutPostRedisplay();
}

void pmousemotion(int x, int y)
{
  startx=x ;
  starty=y ;
  
  view3d.cur.x =  view3d.v.z*tan(dec2rad(view3d.fov/2.0)) * (x     -(view3d.win_w/2))/(view3d.win_h/2) ;
  view3d.cur.y = -view3d.v.z*tan(dec2rad(view3d.fov/2.0)) * (y     -(view3d.win_h/2))/(view3d.win_h/2) ;
  glutPostRedisplay();
}

void init(void)
{
  GLfloat mat_diffuse[] = { 0.7, 0.7, 0.7, 1.0 };
  GLfloat ambient[] = { 0.2,0.2,0.2,1.0 } ;
  
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
  glClearColor(0.6,0.6,0.6,0.0) ;
  glEnable(GL_DEPTH_TEST);
  //  glEnable(GL_CULL_FACE);
  //  glCullFace(GL_BACK);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient ) ;

  theNurb = gluNewNurbsRenderer();
  //  gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 50.0);
  //  gluNurbsProperty(theNurb, GLU_DISPLAY_MODE, GLU_FILL);
  //gluNurbsProperty(theNurb, GLU_DISPLAY_MODE, GLU_OUTLINE_POLYGON);
}

void var_init(void)
{
  view3d.fov  =30.0 ;
  view3d.v.x=0.0   ;  view3d.v.y=0.0   ;  view3d.v.z=5.0   ;
  view3d.c.x=0.0   ;  view3d.c.y=0.0   ;  view3d.c.z=0.0   ;
  view3d.u.x=0.0   ;  view3d.u.y=1.0   ;  view3d.u.z=0.0   ;
  view3d.r.x=0.0   ;  view3d.r.y=0.0   ;  view3d.r.z=0.0   ;
  view3d.p.x=0.0   ;  view3d.p.y=0.0   ;  view3d.p.z=0.0   ;
  view3d.cur.x=0.0 ;  view3d.cur.y=0.0 ;  view3d.cur.z=0.0 ;
  view3d.sta.x=0.0 ;  view3d.sta.y=0.0 ;  view3d.sta.z=0.0 ;
  view3d.win_w=320 ;
  view3d.win_h=240 ;
  view3d.scale=1.0 ;
}

void manprint()
{
  fprintf(stderr,
	  "Motion Designer " VERSION " TANE\n"
	  "Usage: m-designer [-Options]\n"
	  " Options\n"
	  "  -i file or --input-pose=file  : input  pose-format file name\n"
	  "  -o file or --output-pose=file : output pose-format file name\n"
	  "  -h      or --help             : Print this message\n"
	  ) ;
}

int parse_opt(struct GETOPT_ARG *args)
{
  int ret ;
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    {"intpu-pose",         'i', NEEDARG, 'i'},
    {"output-pose",        'o', NEEDARG, 'o'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  
  inputpose  = NULL ;
  outputpose = NULL ;

  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      manprint() ;
      exit(0) ;
      break ;
    case 'i' :
      inputpose=(char*)malloc(strlen(xgetopt_val)+1) ;
      strcpy(inputpose,xgetopt_val) ;
      break ;
    case 'o' :
      outputpose=(char*)malloc(strlen(xgetopt_val)+1) ;
      strcpy(outputpose,xgetopt_val) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      fprintf(stderr,"Option argument not found.\n") ;
      exit(-1) ;
    case XGETOPT_INVALID_OPTION :
      fprintf(stderr,"Invalid option.\n") ;
      exit(-1) ;
    default :
      fprintf(stderr,"xgetopt internal error\n") ;
      exit(-1) ;
    }
    if( ret==XGETOPT_NOT_OPTION ){
    }
  }
  return(0) ;
}
  

int main(int argc, char *argv[])
{
  int i ;
  struct GETOPT_ARG args ;

  XGETOPT_INIARG(args) ;
  for( i=1 ; i<argc ; i++ ) XGETOPT_ADDARG(args,argv[i]) ;
  if( parse_opt(&args)<0 ) exit(-1) ;
  
  var_init();

  if( inputpose ==NULL ) inputpose =default_inputpose ;
  if( outputpose==NULL ) outputpose=default_outputpose ;
  
  yyin=fopen(inputpose,"r") ;
  if( yyin==NULL ){
    printf("Can not open input file(%s).\n",inputpose) ;
    return(-1) ;
  }

  InitMtNode( &M_tree ) ;
  InitPolyObjList( &POlist ) ;
  
  yyparse() ;
  if( yyerrors!=0 ){
    fprintf(stderr, "parse stopped.\n") ;
    exit(-1) ;
  }

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE); /** for DoubleBuffer **/
  glutInitWindowSize(view3d.win_w , view3d.win_h);
  glutCreateWindow(argv[0]);
  glutDisplayFunc(display);
  glutReshapeFunc(resize);
  //  glutIdleFunc(idle); 
  glutKeyboardFunc(key);
  glutSpecialFunc(skey);
  glutMouseFunc(mousebutton);
  glutMotionFunc(mousemotion);
  glutPassiveMotionFunc(pmousemotion);
  init();

  glutMainLoop();
  
  exit(0) ;
}

