#include <stdlib.h>
#include <stdio.h>
#include "obj.h"

int InitMtNode( struct MotionTree *tree )
{
  int i ;
  tree->num=0 ;
  for( i=0 ; i<MAX_CHILD_NUM ; i++ ){
    tree->node[i]=NULL ;
  }
  tree->Mt.p.x=0.0 ;
  tree->Mt.p.y=0.0 ;
  tree->Mt.p.z=0.0 ;
  tree->Mt.r.x=0.0 ;
  tree->Mt.r.y=0.0 ;
  tree->Mt.r.z=0.0 ;
  tree->Mt.obj_id=-1 ;

  return(0) ;
}

int AllocChildMtNode( int nest_depth )
{
  struct MotionTree *tree ;
  int i ;

  tree = &M_tree ;

  for( i=0 ; i<nest_depth+1 ; i++ ){
    if( tree->node[tree->num]==NULL ){
      tree->node[tree->num]=malloc(sizeof(struct MotionTree)) ;
      if( tree->node[tree->num]==NULL ){
	return(-1) ;
      }
      InitMtNode( tree->node[tree->num] ) ;
    }
    tree=tree->node[tree->num] ;
  }
    
  return(0) ;
}

int SetPOSMtNode( int nest_depth, struct Vec3D pos )
{
  struct MotionTree *tree ;
  int i ;

  tree = &M_tree ;

  for( i=0 ; i<nest_depth+1 ; i++ ){
    tree=tree->node[tree->num] ;
  }

  tree->Mt.p=pos ;
    
  return(0) ;
}


int SetROTMtNode( int nest_depth, struct Vec3D rot )
{
  struct MotionTree *tree ;
  int i ;

  tree = &M_tree ;

  for( i=0 ; i<nest_depth+1 ; i++ ){
    tree=tree->node[tree->num] ;
  }

  tree->Mt.r=rot ;
    
  return(0) ;
}


int SetPolyOBJMtNode( int nest_depth, struct PolyObjList *polist, int id )
{
  struct MotionTree *tree ;
  int i ;

  tree = &M_tree ;

  for( i=0 ; i<nest_depth+1 ; i++ ){
    tree=tree->node[tree->num] ;
  }

  tree->Mt.obj_id=id ;
  polist->Obj[id]->node=tree ;
    
  return(0) ;
}


int SetENDMtNode( int nest_depth )
{
  struct MotionTree *tree ;
  int i ;

  tree = &M_tree ;

  for( i=0 ; i<nest_depth ; i++ ){
    tree=tree->node[tree->num] ;
  }

  tree->num++ ;
    
  return(0) ;
}


int AllFreeMtNode( struct MotionTree *tree )
{
  int i ;

  if( tree->num!=0 ){
    for( i=0 ; i<tree->num ; i++ ){
      AllFreeMtNode( tree->node[i] ) ;
      free( tree->node[i] ) ;
    }
  }
  
  return(0) ;
}
  

#define indent(l)  {int n ; for(n=0 ;n<(l)*2;n++)fprintf(ofp," ");}
int DumpMtNode( FILE *ofp, struct MotionTree *tree, int level )
{
  int i ;

  indent(level) ; fprintf(ofp,"obj{\n") ;
  if( tree->Mt.obj_id>=0 ){
    indent(level+1) ; fprintf(ofp,"file=\"%s\" ;\n",POlist.Obj[tree->Mt.obj_id]->fname) ;
  }
  indent(level+1) ;
  fprintf(ofp,"pos=< %f, %f, %f > ;\n",tree->Mt.p.x,tree->Mt.p.y,tree->Mt.p.z) ;
  indent(level+1) ;
  fprintf(ofp,"rot=< %f, %f, %f > ;\n",tree->Mt.r.x,tree->Mt.r.y,tree->Mt.r.z) ;
  
  for( i=0 ; i<tree->num ; i++ ){
    DumpMtNode( ofp, tree->node[i], level+1 ) ;
  }
  indent(level) ; fprintf(ofp,"}\n") ;
  return(0) ;
}


