#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <GL/glut.h>
#include <lib3ds/file.h>
#include <lib3ds/mesh.h>
#include <lib3ds/material.h>
#include <lib3ds/vector.h>
#include <math.h>
#include "obj.h"

struct PolyObjData *read_3ds( char *fname, int id )
{
  Lib3dsFile *f=NULL;
  Lib3dsMesh *m;
  Lib3dsVector pos;
  struct PolyObjData *obj ;
  int points ;
  int faces  ;
  int i ;

  f=lib3ds_file_load(fname);

  if( f==NULL ) return(NULL) ;
  obj=(struct PolyObjData *)malloc(sizeof(struct PolyObjData)) ;
  if( obj==NULL ) return(NULL) ;

  obj->px=0 ; obj->py=0 ; obj->pz=0 ;
  obj->ox=0 ; obj->oy=0 ; obj->oz=0 ;
  obj->drawx=0 ;
  obj->drawy=0 ;
  obj->drawz=0 ;
  obj->plist=NULL ;
  obj->pnum=0 ;
  obj->flist=NULL ;
  obj->fnum=0 ;
  
  for( m=f->meshes ; m!=NULL ; m=m->next ){
    obj->pnum += m->points ;
    obj->fnum += m->faces ;
  }

  obj->plist=(struct Vec3D *)malloc( sizeof(struct Vec3D) * obj->pnum ) ;
  obj->nlist=(struct Vec3D *)malloc( sizeof(struct Vec3D) * obj->pnum ) ;
  obj->flist=(struct Face3D *)malloc( sizeof(struct Face3D) * obj->fnum ) ;
  obj->fname=(char*)malloc( strlen(fname)+1 ) ;
  obj->draw =(int*)malloc( sizeof(int) * obj->pnum ) ;

  if( obj->plist==NULL ||
      obj->nlist==NULL ||
      obj->flist==NULL ||
      obj->fname==NULL ||
      obj->draw ==NULL ){
    if( obj->plist!=NULL ) free(obj->plist) ;
    if( obj->nlist!=NULL ) free(obj->nlist) ;
    if( obj->flist!=NULL ) free(obj->flist) ;
    if( obj->fname!=NULL ) free(obj->fname) ;
    if( obj->draw !=NULL ) free(obj->draw)  ;
    free( obj ) ;
    return(NULL) ;
  }
  for( i=0 ; i<obj->pnum ; i++ ){
    obj->nlist[i].x=0.0 ;
    obj->nlist[i].y=0.0 ;
    obj->nlist[i].z=0.0 ;
    obj->draw[i]   = 0 ;
  }
  strcpy( obj->fname,fname ) ;

  points=0 ;
  faces =0 ;
  for( m=f->meshes ; m!=NULL ; m=m->next ){
    
    for( i=0 ; i<(int)m->points ; i++ ){
      lib3ds_vector_copy(pos, m->pointL[i].pos);
      obj->plist[points+i].x= pos[0] ;
      obj->plist[points+i].y= pos[1] ;
      obj->plist[points+i].z= pos[2] ;
    }
    for( i=0 ; i<(int)m->faces ; i++ ){
      Lib3dsMaterial *mat=0;
      if (m->faceL[i].material) {
	mat=lib3ds_file_material_by_name(f, m->faceL[i].material);
      }
      
      obj->flist[faces+i].p0 = points + m->faceL[i].points[0] ;
      obj->flist[faces+i].p2 = points + m->faceL[i].points[1] ;
      obj->flist[faces+i].p1 = points + m->faceL[i].points[2] ;
      obj->flist[faces+i].normal.x = m->faceL[i].normal[0] ;
      obj->flist[faces+i].normal.y = m->faceL[i].normal[1] ;
      obj->flist[faces+i].normal.z = m->faceL[i].normal[2] ;
      obj->flist[faces+i].id = id ;

      AddVec( &obj->nlist[obj->flist[faces+i].p0],
	      &obj->flist[faces+i].normal,
	      &obj->nlist[obj->flist[faces+i].p0] ) ;
      NormalizeVec( &obj->nlist[obj->flist[faces+i].p0],
		    &obj->nlist[obj->flist[faces+i].p0] ) ;
      AddVec( &obj->nlist[obj->flist[faces+i].p1],
	      &obj->flist[faces+i].normal,
	      &obj->nlist[obj->flist[faces+i].p1] ) ;
      NormalizeVec( &obj->nlist[obj->flist[faces+i].p1],
		    &obj->nlist[obj->flist[faces+i].p1] ) ;
      AddVec( &obj->nlist[obj->flist[faces+i].p2],
	      &obj->flist[faces+i].normal,
	      &obj->nlist[obj->flist[faces+i].p2] ) ;
      NormalizeVec( &obj->nlist[obj->flist[faces+i].p2],
		    &obj->nlist[obj->flist[faces+i].p2] ) ;

      if( mat!=NULL ){
	float  s ;
	objRGBA a={0,0,0,1};
	
	memcpy( obj->flist[faces+i].ambient, a,              sizeof(objRGBA) ) ;
	memcpy( obj->flist[faces+i].diffuse,  mat->diffuse,  sizeof(objRGBA) ) ;
	memcpy( obj->flist[faces+i].specular, mat->specular, sizeof(objRGBA) ) ;
	s = pow(2, 10.0*mat->shininess);
	if (s>128.0) s=128.0 ;
	obj->flist[faces+i].shininess = 128 ;
      }else{
	objRGBA a={0.2, 0.2, 0.2, 1.0};
	objRGBA d={0.8, 0.8, 0.8, 1.0};
	objRGBA s={0.0, 0.0, 0.0, 1.0};
	
	memcpy( obj->flist[faces+i].ambient,  a, sizeof(objRGBA) ) ;
	memcpy( obj->flist[faces+i].diffuse,  d, sizeof(objRGBA) ) ;
	memcpy( obj->flist[faces+i].specular, s, sizeof(objRGBA) ) ;
	obj->flist[faces+i].shininess = 128 ;
      }
    }
    
    points += m->points ;
    faces  += m->faces ;
  }

  printf("file %s has %d faces\n",fname,faces) ;

  lib3ds_file_free(f);

  return(obj) ;
}
  
  
  


