#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <GL/glut.h>
#include <math.h>
#include "obj.h"
#include "poly.h"

int search_node( struct View3D view,  struct MotionTree *tree, struct Node *Nd )
{
  struct Vec3D cur ;
  struct Vec3D eye ;
  struct Vec3D ray ;
  struct Vec3D nray ;
  struct Vec3D x_axis={ 1.0, 0.0, 0.0 } ;
  struct Vec3D y_axis={ 0.0, 1.0, 0.0 } ;
  struct Vec3D z_axis={ 0.0, 0.0, 1.0 } ;
  struct View3D view_stack ;
  int i ;
  
  eye=view.v ;
  cur=view.cur ;
  
  SubVec( &eye, &tree->Mt.p, &eye ) ;
  SubVec( &cur, &tree->Mt.p, &cur ) ;
  RotateVec( &eye, tree->Mt.r.x, &x_axis ) ;
  RotateVec( &eye, tree->Mt.r.y, &y_axis ) ;
  RotateVec( &eye, tree->Mt.r.z, &z_axis ) ;
  RotateVec( &cur, tree->Mt.r.x, &x_axis ) ;
  RotateVec( &cur, tree->Mt.r.y, &y_axis ) ;
  RotateVec( &cur, tree->Mt.r.z, &z_axis ) ;
  
  SubVec( &eye, &cur, &ray ) ;
  NormalizeVec( &ray, &nray ) ;

  //  printf("eye=%f,%f,%f ray=%f,%f,%f\n", eye.x,eye.y,eye.z,
  //	                                ray.x,ray.y,ray.z) ;

  if( tree->Mt.obj_id>=0 ){
    struct Face3D *flist ;
    int            fnum  ;
    struct Vec3D  *plist ;
    struct Vec3D   pos   ;
    
    flist=POlist.Obj[tree->Mt.obj_id]->flist ;
    fnum =POlist.Obj[tree->Mt.obj_id]->fnum  ;
    plist=POlist.Obj[tree->Mt.obj_id]->plist ;

    for( i=0 ; i<fnum ; i++ ){
      int ret ;
      
      ret=Poly_SearchCorssPoint( &eye, &ray,
				 &plist[flist[i].p0], &flist[i].normal,
				 &pos ) ;
      if( i==0 ){
	//	printf("face0=%f,%f,%f\n",
	//	       plist[flist[i].p0].x,
	//	       plist[flist[i].p0].y,
	//	       plist[flist[i].p0].z ) ;
      }
      if( ret==1 ){
	ret=Poly_CheckInside( &plist[flist[i].p0],
			      &plist[flist[i].p1],
			      &plist[flist[i].p2],
			      &pos ) ;
	if( ret==1 ){
	  double       dist ;
	  struct Vec3D tmp ;

	  SubVec( &pos, &eye, &tmp ) ;
	  dist= LengthVec(&tmp) ;
	  if( Nd->d<0.0 || Nd->d > dist ){
	    Nd->d   =dist ;
	    Nd->node=tree ;
	  }
	}
      }
    }
  }

  view_stack    = view ;
  view_stack.v  =eye ;
  view_stack.cur=cur ;
  for( i=0 ; i<tree->num ; i++ ){
    search_node( view_stack, tree->node[i], Nd ) ;
  }
  
  return(0) ;
}

int view_set( struct View3D *view )
{
  struct Vec3D cur ;
  struct Vec3D eye ;
  struct Vec3D x_axis={ 1.0, 0.0, 0.0 } ;
  struct Vec3D y_axis={ 0.0, 1.0, 0.0 } ;
  
  eye=view->v ;
  cur=view->cur ;
  
  SubVec( &eye, &view->p, &eye ) ;
  SubVec( &cur, &view->p, &cur ) ;
  RotateVec( &eye, view->r.x, &x_axis ) ;
  RotateVec( &eye, view->r.y, &y_axis ) ;
  RotateVec( &cur, view->r.x, &x_axis ) ;
  RotateVec( &cur, view->r.y, &y_axis ) ;

  view->v  =eye ;
  view->cur=cur ;
  
  return(0) ;
}
  
