#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include <math.h>
#include "vector.h"

int RotateVec( struct Vec3D *pos, double angle, struct Vec3D *vec )
{
  double sinval, cosval;
  double m[3][3] ;
  double x,y,z ;

  x=vec->x ;
  y=vec->y ;
  z=vec->z ;
  sinval = sin(((M_PI*2)*(360-angle))/360.0);
  cosval = cos(((M_PI*2)*(360-angle))/360.0);
  
  m[0][0] = x * x * (1.0 - cosval) + cosval  ;
  m[0][1] = x * y * (1.0 - cosval) - z * sinval;
  m[0][2] = x * z * (1.0 - cosval) + y * sinval;
  
  m[1][0] = x * y * (1.0 - cosval) + z * sinval;
  m[1][1] = y * y * (1.0 - cosval) + cosval  ;
  m[1][2] = y * z * (1.0 - cosval) - x * sinval;
  
  m[2][0] = x * z * (1.0 - cosval) - y * sinval;
  m[2][1] = y * z * (1.0 - cosval) + x * sinval;
  m[2][2] = z * z * (1.0 - cosval) + cosval  ;


  x=pos->x ;
  y=pos->y ;
  z=pos->z ;
  pos->x=m[0][0]*x + m[0][1]*y + m[0][2]*z ;
  pos->y=m[1][0]*x + m[1][1]*y + m[1][2]*z ;
  pos->z=m[2][0]*x + m[2][1]*y + m[2][2]*z ;

  return(0);
}


int RotateVecRad( struct Vec3D *pos, double angle, struct Vec3D *vec )
{
  double sinval, cosval;
  double m[3][3] ;
  double x,y,z ;

  x=vec->x ;
  y=vec->y ;
  z=vec->z ;
  sinval = sin((M_PI*2)-angle);
  cosval = cos((M_PI*2)-angle);
  
  m[0][0] = x * x * (1.0 - cosval) + cosval  ;
  m[0][1] = x * y * (1.0 - cosval) - z * sinval;
  m[0][2] = x * z * (1.0 - cosval) + y * sinval;
  
  m[1][0] = x * y * (1.0 - cosval) + z * sinval;
  m[1][1] = y * y * (1.0 - cosval) + cosval  ;
  m[1][2] = y * z * (1.0 - cosval) - x * sinval;
  
  m[2][0] = x * z * (1.0 - cosval) - y * sinval;
  m[2][1] = y * z * (1.0 - cosval) + x * sinval;
  m[2][2] = z * z * (1.0 - cosval) + cosval  ;


  x=pos->x ;
  y=pos->y ;
  z=pos->z ;
  pos->x=m[0][0]*x + m[0][1]*y + m[0][2]*z ;
  pos->y=m[1][0]*x + m[1][1]*y + m[1][2]*z ;
  pos->z=m[2][0]*x + m[2][1]*y + m[2][2]*z ;

  return(0);
}


double NormalizeVec(struct Vec3D *va, struct Vec3D *ns)
{
  double dx,dy,dz,d ;
  
  dx = va->x ;
  dy = va->y ;
  dz = va->z ;

  d=sqrt( dx*dx + dy*dy + dz*dz ) ;

  if( d==0.0 ){
    ns->x=0.0 ;
    ns->y=0.0 ;
    ns->z=0.0 ;
  }else{
    ns->x = dx/d ;
    ns->y = dy/d ;
    ns->z = dz/d ;
  }
  return(d) ;
}

double LengthVec(struct Vec3D *va)
{
  double dx,dy,dz,d ;
  
  dx = va->x ;
  dy = va->y ;
  dz = va->z ;

  d=sqrt( dx*dx + dy*dy + dz*dz ) ;

  return(d) ;
}

void AddVec(struct Vec3D *va, struct Vec3D *vb, struct Vec3D *vs)
{
  vs->x = va->x + vb->x ;
  vs->y = va->y + vb->y ;
  vs->z = va->z + vb->z ;
}

void SubVec(struct Vec3D *va, struct Vec3D *vb, struct Vec3D *vs)
{
  vs->x = va->x - vb->x ;
  vs->y = va->y - vb->y ;
  vs->z = va->z - vb->z ;
}

void InvVec(struct Vec3D *va, struct Vec3D *vs)
{
  vs->x = -va->x ;
  vs->y = -va->y ;
  vs->z = -va->z ;
}

void ScaleVec(struct Vec3D *va, double s, struct Vec3D *vs )
{
  vs->x = va->x * s ;
  vs->y = va->y * s ;
  vs->z = va->z * s ;
}

double DotProduct(struct Vec3D *va, struct Vec3D *vb)
{
  return( (va->x * vb->x) + (va->y * vb->y) + (va->z * vb->z) ) ;
}


void CrossProduct(struct Vec3D *va, struct Vec3D *vb, struct Vec3D *vs)
{
  vs->x = va->y * vb->z - va->z * vb->y;
  vs->y = va->z * vb->x - va->x * vb->z;
  vs->z = va->x * vb->y - va->y * vb->x;
}
